.gtkArgs <-
function()
{
 c("R")
}

.onLoad <-
function(libname, pkgname)
{
 options(depwarn = TRUE, gdkFlush = TRUE)
 
 dll <- try(library.dynam("RGtk2", pkgname, libname),
            silent = getOption("verbose"))
 if (is.character(dll)) {
   message("Failed to load RGtk2 dynamic library, attempting to install it.")
   .install_system_dependencies()
   return()
 }
   

 if(is.function(.gtkArgs))
  args <- .gtkArgs()
 else
  args <- as.character(.gtkArgs)

 #if ("REventLoop" %in% rownames(installed.packages())) {
 #    library(REventLoop)
 #    eventLoop(REventLoop("R_Gtk2EventLoop"))
 #    runEventLoop()
 #} else
 if(!(gtkInit(args))) {
   message("R session is headless; GTK+ not initialized.")
   if (length(grep("darwin", R.version$platform)))
     message("Please try running R inside an X11 terminal.")
 }

 .initClasses()
}

.install_system_dependencies <- function()
{
  windows_config <- list(
    source = FALSE,
    gtk_url = "http://downloads.sourceforge.net/gtk-win/gtk2-runtime-2.22.0-2010-10-21-ash.exe?download",
    installer = function(path) {
      shell(path)
    }
  )
  
  darwin_config <- list(
    source = FALSE,
    gtk_url = "http://r.research.att.com/libs/GTK_2.18.5-X11.pkg", 
    installer = function(path) {
      system(paste("open", path))
    }
  )
  
  unix_config <- NULL
  
  gtk_web <- "http://www.gtk.org"
  
  install_system_dep <- function(dep_name, dep_url, dep_web, installer)
  {
    if (!interactive()) {
      message("Please install ", dep_name, " from ", dep_url)
      return()
    }
    choice <- menu(paste(c("Install", "Do not install"), dep_name), T, 
      paste("Need", dep_name, "? (Restart R after installing)"))
    if (choice == 1) {
      path <- file.path(tempdir(), basename(sub("\\?.*", "", dep_url)))
      if (download.file(dep_url, path, mode="wb") > 0)
        stop("Failed to download ", dep_name)
      installer(path)
    }
    message("Learn more about ", dep_name, " at ", dep_web)
  }
  
  install_all <- function() {
    if (.Platform$OS.type == "windows" && .Platform$r_arch == "i386")
      config <- windows_config
    else if (length(grep("darwin", R.version$platform))) 
      config <- darwin_config
    else config <- unix_config
    
    if (is.null(config))
      stop("This platform is not yet supported by the automatic installer. ",
           "Please install GTK+ manually, if necessary. See: ", gtk_web)
    
    install_system_dep("GTK+", config$gtk_url, gtk_web, config$installer)
  }
  
  install_all()
  
  message("If the package still does not load, please ensure that GTK+ is installed and that it is on your PATH environment variable")
  message("IN ANY CASE, RESTART R BEFORE TRYING TO LOAD THE PACKAGE AGAIN")
}
